/*
  BDE.h : BDE score function
  Copyright (C) 2009, 2013 Yoshinori Tamada <tamada  A T  is.s.u-tokyo.ac.jp>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright 
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.

  * Neither the name of The University of Tokyo nor the names of its 
    contributors may be used to endorse or promote products derived 
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __PARAOS_BDE_H
#define __PARAOS_BDE_H

#include <stdio.h>

typedef union BDE_elm_t {
  union BDE_elm_t * elm;
  int value;
} BDEElm;

typedef struct {
  int p;      /* The number of nodes */
  int n;      /* The number of samples */
  int * X; /* (n,p)-fortran style input data matrix for parents */
  int * Y; /* (n,p)-fortran style input data matrix for children */

  /* NOTE: X[i + j * n] corresponds to the (i,j) element in the matrix */

  int prior; /* Prior type. Currently, only BDE_PRIOR_TYPE_FIXED is supported. */
  //int * r;   /* r[i] = # of categories for the i-th variable (node) */
  int r;

  //int * n_ij;  /* working area */
  //int * n_ijk; /* working area */
  //int * j;     /* working area */
  BDEElm ** n_ijk;
  BDEElm * table;
  size_t tpos;

  float fixed_alpha; /* fixed alpha parameter */

  int max_parents;  /* The number of max parents */
  int verbose;      /* verbose mode if non zero. */
  FILE * err;       /* message output file stream */
} BDEBuff;

void *   BDE_init(int * X, int * Y, int p, int n, int c, int m, FILE * fp);
float    BDE_score(const void * buff, const int j, const int * parents, const int q);
void     BDE_finalize(void * buff);

#define Paraos_score(B,j,P,q) BDE_score(B,j,P,q)

#endif /* __PARAOS_BDE_H */
/* End of file.  */
