/*
  Parallel OS Algorithm  Version 9 aka ParaOS-DC.
  Copyright (C) 2016-2017  Yoshinori Tamada  <tamada A T ytlab.jp>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright 
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.

  * Neither the name of The University of Tokyo nor the names of its 
    contributors may be used to endorse or promote products derived 
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

void ytSort_exec_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);

void ytSort_introsort_loop_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, size_t depth_limit);
void ytSort_move_median_to_first_para9_int_pair_t(para9_int_pair_t * result, para9_int_pair_t * a, para9_int_pair_t * b, para9_int_pair_t * c);
para9_int_pair_t * ytSort_pivot_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);
para9_int_pair_t * ytSort_unguarded_partition_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * pivot);

void ytSort_final_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);
void ytSort_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);
void ytSort_unguarded_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);
void ytSort_unguarded_linear_insert_para9_int_pair_t(para9_int_pair_t * last);
para9_int_pair_t * ytSort_move_backward_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * d_last);

void ytSort_make_heap_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last);
void ytSort_adjust_heap_para9_int_pair_t(para9_int_pair_t * first, size_t hole, size_t len, para9_int_pair_t * value);
void ytSort_push_heap_para9_int_pair_t(para9_int_pair_t * first, size_t hole, size_t top, para9_int_pair_t * value);
void ytSort_pop_heap_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * result);

size_t ytSort_lg_para9_int_pair_t(size_t n);
enum {ytSort_threshold_para9_int_pair_t = 16};

void ytSort_exec_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  if(first != last){
  	size_t depth_limit = ytSort_lg_para9_int_pair_t(last - first) * 2;
	ytSort_introsort_loop_para9_int_pair_t(first, last, depth_limit);
	ytSort_final_insertion_sort_para9_int_pair_t(first, last);
  }
}
/* ytSort_exec_para9_int_pair_t() ===================================================== */

size_t ytSort_lg_para9_int_pair_t(size_t n){
  size_t k;
  for(k = 0; n != 1; n >>= 1) ++k;
  return k;
}
/* ytSort_lg_para9_int_pair_t() ======================================================= */

void ytSort_introsort_loop_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, size_t depth_limit){
  while(last - first > (int) ytSort_threshold_para9_int_pair_t){
	if(depth_limit == 0){
	  /* partial_sort(first, last, last); */
	  /*   heap_select(first, last, last); */ 
	  ytSort_make_heap_para9_int_pair_t(first, last);
	  for(para9_int_pair_t * i = last; i < last; ++i){
		if(((i)->b > (first)->b)) ytSort_pop_heap_para9_int_pair_t(first, last, i);
	  }
	  /*   sort_heap() */
	  while(last - first > 1){
		--last;
		ytSort_pop_heap_para9_int_pair_t(first, last, last);
	  }
	  return;
	}

	--depth_limit;
	para9_int_pair_t * cut = ytSort_pivot_para9_int_pair_t(first, last);
	ytSort_introsort_loop_para9_int_pair_t(cut, last, depth_limit);
	last = cut;
  }
}
/* ytSort_introsort_loop_para9_int_pair_t() ============================================= */

para9_int_pair_t * ytSort_pivot_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  para9_int_pair_t * mid = first + (last - first) / 2;
  ytSort_move_median_to_first_para9_int_pair_t(first, first + 1, mid, last - 1);
  return ytSort_unguarded_partition_para9_int_pair_t(first + 1, last, first);
}
/* ytSort_pivot_para9_int_pair_t() */

void ytSort_swap_para9_int_pair_t(para9_int_pair_t * a, para9_int_pair_t * b){
  para9_int_pair_t tmp = *a;
  *a = *b;
  *b = tmp;
}
/* ytSort_swap_para9_int_pair_t() */

void ytSort_move_median_to_first_para9_int_pair_t(para9_int_pair_t * result, para9_int_pair_t * a, para9_int_pair_t * b, para9_int_pair_t * c){
  if(((a)->b > (b)->b)){
	if(((b)->b > (c)->b)){
	  ytSort_swap_para9_int_pair_t(result, b);
	} else if(((a)->b > (c)->b)){
	  ytSort_swap_para9_int_pair_t(result, c);
	} else {
	  ytSort_swap_para9_int_pair_t(result, a);
	}
  } else if(((a)->b > (c)->b)){
	ytSort_swap_para9_int_pair_t(result, a);
  } else if(((b)->b > (c)->b)){
	ytSort_swap_para9_int_pair_t(result, c);
  } else {
	ytSort_swap_para9_int_pair_t(result, b);
  }
}
/* ytSort_move_median_to_first_para9_int_pair_t() */

para9_int_pair_t * ytSort_unguarded_partition_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * pivot){
  while(1){
	while(((first)->b > (pivot)->b)) ++first;
	--last;

	while(((pivot)->b > (last)->b)) --last;
	
	if (!(first < last)) return first;
	ytSort_swap_para9_int_pair_t(first, last);
	++first;
  }
}
/* ytSort_unguarded_partition() ========================================== */

void ytSort_final_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  if(last - first > (int) ytSort_threshold_para9_int_pair_t){
	ytSort_insertion_sort_para9_int_pair_t(first, first + (int) ytSort_threshold_para9_int_pair_t);
	ytSort_unguarded_insertion_sort_para9_int_pair_t(first + (int) ytSort_threshold_para9_int_pair_t, last);
  } else {
	ytSort_insertion_sort_para9_int_pair_t(first, last);
  }
}
/* ytSort_final_insertion_sort_para9_int_pair_t() ======================================= */

void ytSort_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  if(first == last) return;

  for(para9_int_pair_t * i = first + 1; i != last; ++i){
	if(((i)->b > (first)->b)){
	  para9_int_pair_t val = *i;
	  ytSort_move_backward_para9_int_pair_t(first, i, i + 1);
	  *first = val;
	} else {
	  ytSort_unguarded_linear_insert_para9_int_pair_t(i);
	}
  }
}
/* ytSort_insertion_sort_para9_int_pair_t() */

void ytSort_unguarded_insertion_sort_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  for(para9_int_pair_t * i = first; i != last; ++i){
	ytSort_unguarded_linear_insert_para9_int_pair_t(i);
  }
}
/* ytSort_unguarded_insertion_sort_para9_int_pair_t() */

para9_int_pair_t * ytSort_move_backward_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * d_last){
  while(first != last){
	*(--d_last) = *(--last);
  }
  return d_last;
}

void ytSort_unguarded_linear_insert_para9_int_pair_t(para9_int_pair_t * last){
  para9_int_pair_t val = *last;
  para9_int_pair_t * next = last;
  --next;
  while(((&val)->b > (next)->b)){
	*last = *next;
	last = next;
	--next;
  }
  *last = val;
}

void ytSort_make_heap_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last){
  /* If the length is less than 2, then do nothing. */
  if(last - first < 2) return;

  size_t len = last - first;
  size_t parent = (len - 2) / 2;
  /* Here, all values below 'parent' are leaves. */
  
  while(1){
	para9_int_pair_t value = *(first + parent);
	ytSort_adjust_heap_para9_int_pair_t(first, parent, len, &value);
	if(parent == 0) return;
	parent--;
  }
}
/* ytSort_make_heap_para9_int_pair_t() ================================================== */

void ytSort_adjust_heap_para9_int_pair_t(para9_int_pair_t * first, size_t hole, size_t len, para9_int_pair_t * value){
  const size_t top = hole;
  size_t second = hole;
  while(second < (len - 1) / 2){
	second = 2 * (second + 1);
	if(((first + second)->b > (first + (second - 1))->b)) second--;
	*(first + hole) = *(first + second);
	hole = second;
  }

  if((len & 1) == 0 && second == (len - 2) / 2){
	second = 2 * (second + 1);
	*(first + hole) = *(first + (second - 1));
	hole = second - 1;
  }
  
  ytSort_push_heap_para9_int_pair_t(first, hole, top, value);
}
/* ytSort_adjust_heap_para9_int_pair_t() */


/*!\brief Inserts \ref value into the heap.
 *
 *\param first  pointer to the first element of the heap.
 *\param hole  the number of elements including one to be added in the heap.
 *\param top
 *\param value pointer to the value to be added.
 */
void ytSort_push_heap_para9_int_pair_t(para9_int_pair_t * first, size_t hole, size_t top, para9_int_pair_t * value){
  size_t parent = (hole - 1) / 2;
  while(hole > top && ((first + parent)->b > (value)->b)){
	*(first + hole) = *(first + parent);
	hole = parent;
	parent = (hole - 1) / 2;
  }
  *(first + hole) = *value;
}
/* ytSort_push_heap_para9_int_pair_t() */

void ytSort_pop_heap_para9_int_pair_t(para9_int_pair_t * first, para9_int_pair_t * last, para9_int_pair_t * result){
  para9_int_pair_t value = *result;
  *result = *first;
  ytSort_adjust_heap_para9_int_pair_t(first, 0, (last - first), &value);
}
/* ytSort_pop_heap_para9_int_pair_t() */
