/*
  util/ytTimer.{h,c} : Timer functions
  Copyright (C) 2016 Yoshinori Tamada <tamada  A T ytlab.jp>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  * Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

  * Redistributions in binary form must reproduce the above copyright 
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.

  * Neither the name of The University of Tokyo nor the names of its 
    contributors may be used to endorse or promote products derived 
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef __YTLIB_TIMER_H
#define __YTLIB_TIMER_H

#include <sys/time.h>
#include <time.h>
#include <stdio.h>

/*!\brief Timer types.
 *
 *\ingroup Timer */
typedef enum {
  ytTimer_NORMAL,
  ytTimer_CLOCK,
} ytTimer_type_t;

typedef struct ytTimer_t ytTimer;

ytTimer * ytTimer_new();
void    ytTimer_delete(ytTimer * timer);
void    ytTimer_enableSector(ytTimer * this);
void    ytTimer_set_type(ytTimer * timer, ytTimer_type_t type);
ytTimer_type_t
        ytTimer_get_type(const ytTimer * timer);

void    ytTimer_start(ytTimer * timer);
void    ytTimer_start_type(ytTimer * timer, ytTimer_type_t type);
void    ytTimer_stop(ytTimer * timer);
void    ytTimer_stopLap(ytTimer * this);
void    ytTimer_clear(ytTimer * timer);
void    ytTimer_clearStart(ytTimer * timer);

double  ytTimer_get(const ytTimer * timer);
double  ytTimer_lap(ytTimer * timer);
void    ytTimer_sector(ytTimer * this);
double  ytTimer_get_avg_lap(const ytTimer * timer);
double  ytTimer_getTime();
void    ytTimer_print_DDHHMMSS(ytTimer * timer, FILE * fp);
void    ytTimer_print_DDHHMMSS_sec(FILE * fp, double seconds);
void    ytTimer_print_DDHHMMSSSSS_sec(FILE * fp, double seconds);
void    ytTimer_print_DDHHMMSSSSS(const ytTimer * timer, FILE * fp);
void    ytTimer_print(const ytTimer * this, FILE * fp);

#endif /* __YTLIB_TIMER_H */

/* End of file. */
